<?xml version="1.0" encoding="UTF-8" standalone="no"?>
<svg version="1.1" xmlns="http://www.w3.org/2000/svg"
     xmlns:xi="http://www.w3.org/2001/XInclude"
     width="layout_width"
     height="layout_height" >
  <style type="text/css">
    .Leader {
      stroke-width:  .5 !important;
      stroke:   dark_color;
      fill:        none !important;
    }
    .Button {
      stroke-width: .25;
      stroke:   inactive_color;
      fill:     inactive_color;
    }
    buttons_section
    .Leader {
      fill:        none !important;
    }
    .Label {
      stroke:      none !important;
      stroke-width:  .1 !important;
      font-size:     .1 !important;
      fill:     back_color !important;
    }
    .TouchStrip,.TouchRing {
      stroke-width:  .1 !important;
      stroke:   inactive_color !important;
      fill:     dark_color !important;
    }
  </style>
  <xi:include href="layout_file" />
</svg>
